/****************************************************************************

 MESSAGE PASSING INTERFACE TEST CASE SUITE

 Copyright IBM Corp. 1995

 IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 distribute this software for any purpose and without fee provided that the
 above copyright notice and the following paragraphs appear in all copies.

 IBM Corp. makes no representation that the test cases comprising this
 suite are correct or are an accurate representation of any standard.

 In no event shall IBM be liable to any party for direct, indirect, special
 incidental, or consequential damage arising out of the use of this software
 even if IBM Corp. has been advised of the possibility of such damage.

 IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

****************************************************************************

 These test cases reflect an interpretation of the MPI Standard.  They are
 are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 test case from this set believes that the MPI Standard requires behavior
 different than that implied by the test case we would appreciate feedback.

 Comments may be sent to:
    Richard Treumann
    treumann@kgn.ibm.com

****************************************************************************
*/
//#include "mpi.h"
import mpi.*;
 
//int main( argc, argv )
//int  argc;
//char **argv;
class group {
  static public void main(String[] args) {

    int tasks,me,size,rank,i,result,rc;//,class;
    //int ranks1[128],ranks2[128];
    //MPI_Group group1,group2,group3,newgroup;
    //MPI_Group groups[20];
    int       cnt = 0;
    //MPI_Comm newcomm;
    
    //MPI_Init(&argc,&argv);
    MPI.Init(args);
    
    Group group1,group2,group3,newgroup;
    Group groups[] = new Group[20];
    Comm newcomm;
    
    //MPI_Comm_size(MPI_COMM_WORLD,&tasks);
    tasks = MPI.COMM_WORLD.Size();
    int ranks1[] = new int[tasks/2];
    int ranks2[] = new int[tasks/2];
    int ranks3[] = new int[tasks];
    
    if(tasks < 2) { 
      System.out.println("MUST HAVE AT LEAST 2 TASKS"); 
      System.exit(0); }
    
    //MPI_Comm_rank(MPI_COMM_WORLD,&me);
    me = MPI.COMM_WORLD.Rank();
    
    //MPI_Comm_group(MPI_COMM_WORLD,&group1);
    group1 = MPI.COMM_WORLD.Group();
    groups[cnt++] = group1;
    
    //MPI_Group_size(group1,&size);
    size = group1.Size();
    
    if(size != tasks)  
      System.out.println
	("ERROR in MPI_Group_size, size = "+size+", should be "+tasks);

   //MPI_Group_rank(group1,&rank);
    rank = group1.Rank();

    if(rank != me)  
      System.out.println
	("ERROR in MPI_Group_rank, rank = "+rank+", should be "+me);

    for(i=0;i<tasks/2;i++)  ranks1[i] = i;

    //MPI_Group_incl(group1,tasks/2,ranks1,&newgroup);
    newgroup = group1.Incl(ranks1);

    /* newgroup freed below */
    //MPI_Group_size(newgroup,&size);
    size = newgroup.Size();
    if(size != (tasks/2))  
      System.out.println
	("ERROR in MPI_Group_size, size = "+size+", should be "+(tasks/2));


   //MPI_Group_compare(newgroup,newgroup,&result);
    result = Group.Compare(newgroup,newgroup);
    if(result != MPI.IDENT)  
      System.out.println
	("ERROR in MPI_Group_compare (1), result = "+result+
	 ", should be "+MPI.IDENT);

    //MPI_Group_compare(newgroup,group1,&result);
    result = Group.Compare(newgroup,group1);
    if(result != MPI.UNEQUAL)  
      System.out.println
	("ERROR in MPI_Group_compare (2), result = "+result+
	 ", should be "+MPI.UNEQUAL);

    //MPI_Group_union(group1,newgroup,&group2);
    group2 = Group.Union(group1,newgroup);
    groups[cnt++] = group2;
    //MPI_Group_compare(group1,group2,&result);
    result = Group.Compare(group1,group2);
    if(result != MPI.IDENT)  
      System.out.println
	("ERROR in MPI_Group_compare (3), result = "+result+
	 ", should be "+MPI.IDENT);

    //MPI_Group_intersection(newgroup,group1,&group2);
    group2 = Group.Intersection(newgroup,group1);
    groups[cnt++] = group2;
    //MPI_Group_compare(group2,newgroup,&result);
    result = Group.Compare(group2,newgroup);
    if(result != MPI.IDENT)  
      System.out.println
	("ERROR in MPI_Group_compare (4), result = "+result+
	 ", should be "+MPI.IDENT);
   
    //MPI_Group_difference(group1,newgroup,&group2);
    group2 = Group.Difference(group1,newgroup);
    groups[cnt++] = group2;
    //MPI_Group_size(group2,&size);
    size = group2.Size();
    if(size != (tasks/2))
      System.out.println
	("ERROR in MPI_Group_size, size = "+size+
	 ", should be "+(tasks/2));

    for(i=0;i<size;i++)  ranks1[i] = i;
    //MPI_Group_translate_ranks(group2,size,ranks1,group1,ranks2);
    ranks2 = Group.Translate_ranks(group2,ranks1,group1);
    for(i=0;i<size;i++) {
      if(ranks2[i] != (tasks/2 + i))  
	System.out.println("ERROR in MPI_Group_translate_ranks.");
    }

    
    //MPI_Comm_create(MPI_COMM_WORLD,newgroup,&newcomm);
    newcomm = MPI.COMM_WORLD.Creat(newgroup);
    //if(newcomm != MPI_COMM_NULL)  {
    if(newcomm != null)  {
      //MPI_Comm_group(newcomm,&group3);
      group3 = newcomm.Group();
      groups[cnt++] = group3;
      //if(group3 != newgroup)  <-- wrong 
      result = Group.Compare(group3,newgroup);
      if(result != MPI.IDENT)  
	// System.out.println
	//  ("ERROR in MPI_Comm_group, group = "+group2+
	//   ", should be "+newgroup); 
      System.out.println
	("ERROR in MPI_Group_compare (4.5) , result = "+result+
	 ", should be "+MPI.IDENT);
  }
  
    //MPI_Group_excl(group1,tasks/2,ranks1,&group3);
    group3 = group1.Excl(ranks1);
    groups[cnt++] = group3;
    //MPI_Group_compare(group2,group3,&result);
    result = Group.Compare(group2,group3);
    if(result != MPI.IDENT)  
      System.out.println
	("ERROR in MPI_Group_compare (5) , result = "+result+
	 ", should be "+MPI.IDENT);

    // ranks1 size is tasks/2
    //for(i=0;i<tasks;i++)  ranks1[tasks-1-i] = i;
    for(i=0;i<tasks;i++)  ranks3[tasks-1-i] = i;
    //MPI_Group_incl(group1,tasks,ranks1,&group3);
    group3 = group1.Incl(ranks3);
    groups[cnt++] = group3;
    //MPI_Group_compare(group1,group3,&result);
    result = Group.Compare(group1,group3);
    if(result != MPI.SIMILAR)  
      System.out.println
	("ERROR in MPI_Group_compare (6), result = "+result+
	 ", should be "+MPI.SIMILAR);
    
	

   /*vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
   group3 = newgroup;
   MPI_Group_free(&newgroup);
   if(newgroup != MPI_GROUP_NULL)  
     System.out.println
       ("ERROR in MPI_Group_free, group = %d, should be %d\n",newgroup,MPI_GROUP_NULL);
   if(newcomm != MPI_COMM_NULL)  {
      MPI_Comm_free(&newcomm);
      if(newcomm != MPI_COMM_NULL)  
	System.out.println
	  ("ERROR in MPI_Comm_free, comm = %d, should be %d\n",newcomm,MPI_COMM_NULL);
   }
   MPI_Errhandler_set(MPI_COMM_WORLD,MPI_ERRORS_RETURN);
   rc = MPI_Comm_create(MPI_COMM_WORLD,newgroup,&newcomm);  
   MPI_Error_class(rc,&class);
   if(class != MPI_ERR_GROUP)  
     System.out.println
       ("ERROR in MPI_Group_free, group "+newgrou+" not freed");
   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/

   //MPI_Barrier(MPI_COMM_WORLD);
    MPI.COMM_WORLD.Barrier();
    if(me == 0)  System.out.println("Group TEST COMPLETE\n"); 

   /* Note newcomm above is not actually created... */
   /*vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
   for (i=0; i<cnt; i++) 
     MPI_Group_free( &groups[i] );
   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/

    MPI.Finalize();

  }
}
